/* binio.c */

/*
 * The file format is assumed to be BIG-ENDIAN.
 * If this code is compiled with -DLITTLE and executes on a little endian
 * CPU then byte-swapping will be done.
 *
 * If an ANSI compiler is used prototypes and ANSI function declarations
 * are used.  Otherwise use K&R conventions.
 *
 * If we're running on a CRAY (8-byte ints and floats), conversions will
 * be done as needed.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#ifdef cray
#  include <string.h>
#endif
#include "jser.h"


/**********************************************************/
/*                  Byte Flipping                         */
/**********************************************************/


#define FLIP4( n )  (  (n & 0xff000000) >> 24		\
		     | (n & 0x00ff0000) >> 8		\
		     | (n & 0x0000ff00) << 8		\
		     | (n & 0x000000ff) << 24  )


/*
 * Flip the order of the 4 bytes in an array of 4-byte words.
 */
#ifdef __STDC__
  void flip4( unsigned int *src, unsigned int *dest, int n )
#else
  void flip4( src, dest, n )
  unsigned int *src, *dest;
  int n;
#endif
{
   int i;
   for (i=0;i<n;i++) {
      register unsigned int tmp = src[i];
      dest[i] = FLIP4( tmp );
   }
}


#ifdef cray

/** THESE ROUTINES MUST BE COMPILED ON THE CRAY ONLY SINCE THEY **/
/** REQUIRE 8-BYTES PER C-TYPE LONG                             **/

/* Cray to IEEE single precision */
static void c_to_if(t,f)
long *t,*f;
{
    if (*f != 0){
        *t = (((*f & 0x8000000000000000) |      /* sign bit */
                 ((((*f & 0x7fff000000000000) >> 48)-16258) << 55)) + /* exp */
                 (((*f & 0x00007fffff000000) +
                    ((*f & 0x0000000000800000) << 1)) << 8));  /* mantissa */
    }
    else *t = *f;
}


#define C_TO_IF( T, F )							\
	if (F != 0) {							\
		T = (((F & 0x8000000000000000) |			\
		((((F & 0x7fff000000000000) >> 48)-16258) << 55)) +	\
		(((F & 0x00007fffff000000) +				\
		((F & 0x0000000000800000) << 1)) << 8));		\
	}								\
	else {								\
		T = F;							\
	}



/* IEEE single precison to Cray */
static void if_to_c(t,f)
long *t,*f;
{
    if (*f != 0) {
        *t = (((*f & 0x8000000000000000) |
                ((*f & 0x7f80000000000000) >> 7) +
                (16258 << 48)) |
                (((*f & 0x007fffff00000000) >> 8) | (0x0000800000000000)));
        if ((*f << 1) == 0) *t = 0;
    }
    else *t = *f;
}

/* T and F must be longs! */
#define IF_TO_C( T, F )							\
	if (F != 0) {							\
		T = (((F & 0x8000000000000000) |			\
		((F & 0x7f80000000000000) >> 7) +			\
		(16258 << 48)) |					\
		(((F & 0x007fffff00000000) >> 8) | (0x0000800000000000)));  \
		if ((F << 1) == 0) T = 0;				\
	}								\
	else {								\
		T = F;							\
	}

#endif /*cray*/



/**************************************************************/
/*                     Read Functions                         */
/**************************************************************/

/*
 * receive_data 
 *  Receive a block of data by repeatedly reading from a socket.  This
 *  function will block the process if data is not available.
 *  Input:  socket - socket to read from
 *           buffer - pointer to data buffer
 *          bytes - how many bytes to read into buffer
 *  Return:  number of bytes received.  This will always either be
 *           the same as the bytes requested or 0 if the connection
 *           was lost.
 */
int receive_data( f, buffer, bytes )
int f;
char *buffer;
int bytes;
{
   int sofar, remaining, len;

   sofar = 0;
   remaining = bytes;
   do {
      len = read( f, buffer + sofar, remaining);
      if (len<=0) return 0;
      sofar += len;
      remaining -= len;
   } while (remaining > 0);
   return bytes;
}


/*
 * Read a 4-byte integer. This function will
 *  block if the data is not available.
 * Input:  f - the file descriptor to read from
 *         i - pointer to integer to put result into.
 * Return:  1 = ok, 0 = error
 */
#ifdef __STDC__
  int read_int4( int f, int *i )
#else
  int read_int4( f, i, 4 )
  int f;
  int *i;
#endif
{
#ifdef LITTLE
   /* read big endian and convert to little endian */
   unsigned int n;
   if (receive_data( f, &n, 4 )==4) {
      *i = FLIP4( n );
      return 1;
   }
   else {
      return 0;
   }
#else
   if (receive_data( f, i, 4 )==4) {
#  ifdef cray
      *i = *i >> 32;
#  endif
      return 1;
   }
   else {
      return 0;
   }
#endif
}


/*
 * Read a 4-byte IEEE float.
 * Input:  f - the file descriptor to read from.
 *         x - pointer to float to put result into.
 * Return:  1 = ok, 0 = error
 */
#ifdef __STDC__
  int read_float4( int f, float *x )
#else
  int read_float4( f, x )
  int f;
  float *x;
#endif
{
#ifdef cray
   long buffer = 0;

   if ( receive_data( f, &buffer, 4 )==4 ) {
      /* convert IEEE float (buffer) to Cray float (x) */
      if_to_c( x, &buffer );
      return 1;
    }
    return 0;
#else
#  ifdef LITTLE
      unsigned int n, *iptr;
      if (receive_data( f, &n, 4 )==4) {
	 iptr = (unsigned int *) x;
	 *iptr = FLIP4( n );
	 return 1;
      }
      else {
	 return 0;
      }
#  else
      if (receive_data( f, x, 4 )==4) {
	 return 1;
      }
      else {
	 return 0;
      }
#  endif
#endif
}


/********************************************************/
/*                         Write Functions              */
/********************************************************/

/*
 * Write a 4-byte integer.
 *Input:  f - the file descriptor
 *         i - the integer
 * Return:  1 = ok, 0 = error
 */
#ifdef __STDC__
  int write_int4( int f, int i )
#else
  int write_int4( f, i )
  int f;
  int i;
#endif
{
#ifdef cray
   i = i << 32;
   return write( f, &i, 4 ) > 0;
#else
#  ifdef LITTLE
     i = FLIP4( i );
#  endif
   return write( f, &i, 4 ) > 0;
#endif
}


/*
 * Write a 4-byte IEEE float.
 * Input:  f - the file descriptor
 *         x - the float
 * Return:  1 = ok, 0 = error
 */
#ifdef __STDC__
  int write_float4( int f, float x )
#else
  int write_float4( f, x )
  int f;
  float x;
#endif
{
#ifdef cray
   char buffer[8];
   c_to_if( buffer, &x );
   return write( f, buffer, 4 ) > 0;
#else
#  ifdef LITTLE
      float y;
      unsigned int *iptr = (unsigned int *) &y, temp;
      y = (float) x;
      temp = FLIP4( *iptr );
      return write( f, &temp, 4 ) > 0;
#  else
      float y;
      y = (float) x;
      return write( f, &y, 4 ) > 0;
#  endif
#endif
}

/* java_ser1d.c */

#include <stdio.h>
#include <stdlib.h>
#include "jser.h"
#include "jser.h"

/*********************************************************
 * java_ser: connect to server and send the data
 *     
 *
 * return value (all functions)
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 *********************************************************/



/*************************************************************
 * java_fser_
 * c program to link with fortran 90
 * This program writes the x and fx values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 ************************************************************/
/* ONLY SUPPORT BIG ENDIAN */
int java_fser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   size = *nx;
   
   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&fx[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser_
 * c program to link with fortran 90
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_ser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 10;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)fx[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_mser_(name, nt, time, x, fx, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double fx[];
  int nx[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, i, j, index;
   int command = 12;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)fx[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_cmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   fx   - fx value array
 *   cx   - color value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_cmser_(name, nt, time, x, fx, cx, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double fx[];
  double cx[];
  int nx[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, i, j, index;
   int command = 20;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)fx[index]);
         write_float4(sockfd, (float)cx[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbser_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   bb    - x min and max  value
 *   xmax    - x max value
 *   nx      - number of x points 
 *   fx     - fxy value array
 ************************************************************/
int java_bbser_(name, time, bb, nx, fx) 
  char *name;
  double *time;
  double bb[];
  int *nx;
  double fx[];
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 28;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_float4(sockfd, (float)bb[0]);
   write_float4(sockfd, (float)bb[1]);



   /* write fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)fx[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   bb    - x min and max array
 *   nx    -   nx value array
 *   fx   - fxy value array
 ************************************************************/
int java_bbmser_(name, nt, time, bb, nx, fx) 
  char *name;
  int *nt;
  double time[];
  double bb[][4];
  int nx[];
  double fx[];

{
   int sockfd;
   int ntsize = *nt, name_length, i, j, index;
   int command = 29;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);


   index = 0;
   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      write_float4(sockfd, (float)bb[i][0]);
      write_float4(sockfd, (float)bb[i][1]);
     
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)fx[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_pser_
 * c program to link with fortran 90
 * to be used for parallel machine
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *   p    - total number of processors
 ************************************************************/
int java_pser_(name, time, x, fx, nx, p) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
  int *p;
{
   int sockfd;
   int size = *nx, name_length, i, pp = *p;
   double ft = *time;
   float ftime = (float)ft;
   int command = 17;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time, nx, and p */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_int4(sockfd, pp);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)fx[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_vser_
 * c program to link with fortran 90
 * This program writes the vector data to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - r value array
 *   nx   - size of x, y, r, t array
 ************************************************************/
int java_vser_(name, time, x, y, r, t, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double r[];
  double t[];
  int *nx;
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 22;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)r[i]);
      write_float4(sockfd, (float)t[i]);
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_vmser_
 * c program to link with fortran 90
 * sends multiple vector data to the server
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - t value array
 *   nx   - size of x, y, r, t array
 ************************************************************/
int java_vmser_(name, nt, time, x, y, r, t, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double r[];
  double t[];
  int nx[];
{
   int sockfd;
   int ntsize = *nt, name_length, i, j, index;
   int command = 27;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)y[index]);
         write_float4(sockfd, (float)r[index]);
         write_float4(sockfd, (float)t[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_sercon_
 * c program to link with fortran 90
 * This program writes the vector data to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   c    - color value array
 *   nx   - size of x, y, c array
 *   cl   - connection list
 *   nc   - the number of elements of connection list
 ************************************************************/
int java_sercon_(name, time, x, y, c, nx, cl, nc) 
  char *name;
  double *time;
  double x[];
  double y[];
  double c[];
  int *nx;
  int cl[];
  int *nc;
{
   int sockfd;
   int size = *nx, con = *nc, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 19;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx, nc */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_int4(sockfd, con);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)c[i]);
   }

   for (i=0; i<con; i++) 
      write_int4(sockfd, cl[i]);

   
   close(sockfd);
   return 0;
}





int java_ser(char *name, double time, double x[], double y[], int n) {
   return java_ser_(name, &time, x, y, &n);
}

int java_mser(char *name, int nt, double time[], double x[], double fx[], int nx[]) {
   return java_mser_(name, &nt, time, x, fx, nx);
}

int java_cmser(char *name, int nt, double time[], double x[], double fx[], double cx[], int nx[]) {
   return java_cmser_(name, &nt, time, x, fx, cx, nx);
}


int java_bbser(char *name, double time, double bb[], int nx, double fx[]) {
  java_bbser_(name, &time, bb, &nx, fx);
}

int java_bbmser(char *name, int nt, double time[], double bb[][4], int nx[], double fx[]) {
   java_bbmser_(name, &nt, time, bb, nx, fx);
}

int java_pser(char *name, double time, double x[], double fx[], int nx, int p) {
   java_pser_(name, &time, x, fx, &nx, &p);
}


int java_vser(char *name, double time, double x[], double y[], double r[], double t[], int nx) {
   java_vser_(name, &time, x, y, r, t, &nx);
}

java_vmser(char *name, int nt, double time[], double x[], double y[], double r[], double t[], int nx[]) {
   java_vmser_(name, &nt, time, x, y, r, t, nx);
}


int java_sercon(char *name, double time, double x[], double y[], double c[], int nx, int cl[], int nc) {
   java_sercon_(name, &time, x, y, c, &nx, cl, &nc);
}
/* java_ser2d.c */

#include <stdio.h>
#include <stdlib.h>
#include "jser.h"
#include "jser.h"

/*********************************************************
 * java_ser2d: connect to server and send the data
 *     
 *
 * return value (all functions)
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 *********************************************************/


/*************************************************************
 * java_fser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x, y and fx array
 ************************************************************/
/* ONLY SUPPORT BIG ENDIAN */
int java_fser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }
   size = *nx;

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&y[i], sizeof(double), 1, fp);
      fwrite(&fxy[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x and fx array
 ************************************************************/

int java_ser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int sockfd;
   int size = *nx, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 15;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and nx */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   fxy   - fxy value array
 *   nx   - size of x and fx array
 ************************************************************/
int java_mser2d_(name, nt, time, x, y, fxy, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double fxy[];
  int nx[];
{

   int sockfd;
   int ntsize = *nt;
   int name_length, i, j, index;
   int command = 16;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[index]);
         write_float4(sockfd, (float)y[index]);
         write_float4(sockfd, (float)fxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 * (reduced form of java_ser2d)
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   nx   - x array size
 *   y    - y value array
 *   ny   - y array size
 *   fxy  - fxy value array
 ************************************************************/
int java_bser2d_(name, time, x, nx, y, ny, fxy) 
  char *name;
  double *time;
  double x[];
  int *nx;
  double y[];
  int *ny;
  double fxy[];
{
   int sockfd;
   int size, xsize = *nx, ysize = *ny, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 30;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time */
   write_float4(sockfd, ftime);

   /* send x array */
   write_int4(sockfd, xsize);
   for (i=0; i<xsize; i++) 
      write_float4(sockfd, (float)x[i]);

   /* send y array */
   write_int4(sockfd, ysize);
   for (i=0; i<ysize; i++) 
      write_float4(sockfd, (float)y[i]);

   /* send fxy array */
   size = xsize * ysize;
   write_int4(sockfd, size);
   for (i=0; i<size; i++) 
      write_float4(sockfd, (float)fxy[i]);

   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bmser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   nx   - size of x array
 *   y    - y value array
 *   ny   - size of y array 
 *   fxy   - fxy value array
 ************************************************************/
int java_bmser2d_(name, nt, time, x, nx, y, ny, fxy) 
  char *name;
  int *nt;
  double time[];
  double x[];
  int nx[];
  double y[];
  int ny[];
  double fxy[];
{
  
   int sockfd;
   int ntsize = *nt, size, name_length, i, j, index, xindex, yindex;
   int command = 31;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   /* write time data */
   index = 0;
   xindex = 0;
   yindex = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);

      write_int4(sockfd, nx[i]);
      for (j=0; j<nx[i]; j++) {
         write_float4(sockfd, (float)x[xindex]);
         xindex++;
      }

      write_int4(sockfd, ny[i]);
      for (j=0; j<ny[i]; j++) {
         write_float4(sockfd, (float)y[yindex]);
         yindex++;
      }

      size = nx[i] * ny[i];
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)fxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}

/*************************************************************
 * java_bbser2d_
 * c program to link with fortran 90
 * sends 2d bounding box and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   bb    - x min, max, y min, and y max
 *   xmax    - x max value
 *   nx      - number of x points 
 *   ymin    - y min value
 *   ymax    - y max value
 *   ny      - number of y points 
 *   fxy     - fxy value array
 ************************************************************/
int java_bbser2d_(name, time, bb, nx, ny, fxy) 
  char *name;
  double *time;
  double bb[];
  int *nx;
  int *ny;
  double fxy[];
{
   int sockfd;
   int size, xsize = *nx, ysize = *ny, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int command = 25;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   size = xsize * ysize;
   /* write time, x and y size, and bb */
   write_float4(sockfd, ftime);
   write_int4(sockfd, xsize);
   write_int4(sockfd, ysize);
   write_float4(sockfd, (float)bb[0]);
   write_float4(sockfd, (float)bb[1]);
   write_float4(sockfd, (float)bb[2]);
   write_float4(sockfd, (float)bb[3]);

   /* write fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}

/*************************************************************
 * java_bbmser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   bb    - x min, max, y min, and y max value array
 *   nx    -   nx value array
 *   ny    -   ny value array
 *   fxy   - fxy value array
 ************************************************************/
int java_bbmser2d_(name, nt, time, bb, nx, ny, fxy) 
  char *name;
  int *nt;
  double time[];
  double bb[][4];
  int nx[];
  int ny[];
  double fxy[];

{
   int sockfd;
   int ntsize = *nt, size, name_length, i, j, index;
   int command = 26;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   index = 0;
   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, time[i]);
      write_int4(sockfd, nx[i]);
      write_int4(sockfd, ny[i]);
      write_float4(sockfd, (float)bb[i][0]);
      write_float4(sockfd, (float)bb[i][1]);
      write_float4(sockfd, (float)bb[i][2]);
      write_float4(sockfd, (float)bb[i][3]);
     
      size = nx[i] * ny[i];
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)fxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_cbbmser2d_
 * sends data like java_bbmser2d.
 * It also sends the color data to the server. Each point can have 
 * a custom color with the specified RGB value, where the red component 
 * is in bits 16-23 of the argument, the green component is in bits 8-15 
 * of the argument, and the blue component is in bits 0-7. The value zero
 * indicates no contribution from the primary color component. 
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   bb    - x min, max, y min, and y max value array
 *   nx    -   nx value array
 *   ny    -   ny value array
 *   fxy   - fxy value array
 *   cxy   - color value array
 ************************************************************/
int java_cbbmser2d_(name, nt, time, bb, nx, ny, fxy, cxy) 
  char *name;
  int *nt;
  double time[];
  double bb[][4];
  int nx[];
  int ny[];
  double fxy[];
  double cxy[];

{
   int sockfd;
   int ntsize = *nt, size, name_length, i, j, index;
   int command = 33;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* send nt */
   write_int4(sockfd, ntsize);

   index = 0;
   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, time[i]);
      write_int4(sockfd, nx[i]);
      write_int4(sockfd, ny[i]);
      write_float4(sockfd, (float)bb[i][0]);
      write_float4(sockfd, (float)bb[i][1]);
      write_float4(sockfd, (float)bb[i][2]);
      write_float4(sockfd, (float)bb[i][3]);
     
      size = nx[i] * ny[i];
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)fxy[index]);
         write_float4(sockfd, (float)cxy[index]);
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_pser_
 * c program to link with fortran 90
 * to be used for parallel machine
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *   p    - total number of processors
 ************************************************************/
int java_pser2d_(name, time, x, y, fxy, nx, p) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
  int *p;
{
   int sockfd;
   int size = *nx, name_length, i, pp = *p;
   double ft = *time;
   float ftime = (float)ft;
   int command = 18;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time, nx, and p */
   write_float4(sockfd, ftime);
   write_int4(sockfd, size);
   write_int4(sockfd, pp);

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)x[i]);
      write_float4(sockfd, (float)y[i]);
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}



/*************************************************************
 * java_pbbser2d_
 * sends 2d bounding box and fxy values to socket.
 * used by parallel machine
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   bb    - x min, max, y min, and y max
 *   xmax    - x max value
 *   nx      - number of x points 
 *   ymin    - y min value
 *   ymax    - y max value
 *   ny      - number of y points 
 *   fxy     - fxy value array
 *   p       - the number of total processors
 ************************************************************/
int java_pbbser2d_(name, time, bb, nx, ny, fxy, p) 
  char *name;
  double *time;
  double bb[];
  int *nx;
  int *ny;
  double fxy[];
  int *p;
{
   int sockfd;
   int size, xsize = *nx, ysize = *ny, name_length, i;
   double ft = *time;
   float ftime = (float)ft;
   int pp = *p;
   int command = 32;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);
   
   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   size = xsize * ysize;
   /* write time, x and y size, and bb */
   write_float4(sockfd, ftime);
   write_int4(sockfd, xsize);
   write_int4(sockfd, ysize);
   write_float4(sockfd, (float)bb[0]);
   write_float4(sockfd, (float)bb[1]);
   write_float4(sockfd, (float)bb[2]);
   write_float4(sockfd, (float)bb[3]);
   write_int4(sockfd, pp);

   /* write fx value */
   for (i=0; i<size; i++) {
      write_float4(sockfd, (float)fxy[i]);
   }
   
   close(sockfd);
   return 0;
}



int java_ser2d(char *name, double time, double x[], double y[], double fxy[], int n) {
   return java_ser2d_(name, &time, x, y, fxy, &n);
}

int java_mser2d(char *name, int nt, double time[], double x[], double y[], double fxy[], int nx[]) {
   return java_mser2d_(name, &nt, time, x, y, fxy, nx);
}

int java_bser2d(char *name, double time, double x[], int nx, double y[], int ny, double fxy[]) {
  java_bser2d_(name, &time, x, &nx, y, &ny, fxy);
}

int java_bmser2d(char *name, int nt, double time[], double x[], int nx[], double y[], int ny[], double fxy[]) {
  java_bmser2d_(name, &nt, time, x, nx, y, ny, fxy);
}

int java_bbser2d(char *name, double time, double bb[], int nx, int ny, double fxy[]) {
  java_bbser2d_(name, &time, bb, &nx, &ny, fxy);
}

int java_bbmser2d(char *name, int nt, double time[], double bb[][4], int nx[], int ny[], double fxy[]) {
   java_bbmser2d_(name, &nt, time, bb, nx, ny, fxy);
}

int java_cbbmser2d(char *name, int nt, double time[], double bb[][4], int nx[], int ny[], double fxy[], double cxy[]) {
   java_cbbmser2d_(name, &nt, time, bb, nx, ny, fxy, cxy);
}


int java_pser2d(char *name, double time, double x[], double y[], double fx[], int nx, int p) {
   java_pser2d_(name, &time, x, y, fx, &nx, &p);
}


int java_pbbser2d(char *name, double time, double bb[], int nx, int ny, double fxy[], int p) {
   java_pbbser2d_(name, &time, bb, &nx, &ny, fxy, &p);
}



/* java_ser3d.c */

#include <stdio.h>
#include <stdlib.h>
#include "jser.h"
#include "jser.h"


/*************************************************************
 * java_ser3d_
 * c program to link with fortran 90
 * This program writes the xsize, ysize, zsize and z values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   nx    - x dimension 
 *   ny    - y dimension 
 *   nz    - z dimension 
 *   threshold - initial threshold
 *   z   - z array
 ************************************************************/

int java_ser3d_(name, time, nx, ny, nz, threshold, grid) 
  char *name;
  double *time;
  int *nx;
  int *ny;
  int *nz; 
  double *threshold;
  double grid[];
{
   int sockfd;
   int nnx = *nx, nny = *ny, nnz = *nz, name_length, i;
   int size = nnx * nny * nnz;
   double ft = *time, thre = *threshold;
   float thresh = (float)thre;
   float ftime = (float)ft;
   int command = 1;

   sockfd = initialize2();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, 1);     /* send no of time */
   write_float4(sockfd, thresh);
   write_int4(sockfd, size);

   write_float4(sockfd, ftime);
   write_int4(sockfd, nnx);
   write_int4(sockfd, nny);
   write_int4(sockfd, nnz);

   /* write x and fx value */
   for (i=0; i<size; i++) 
      write_float4(sockfd, (float)grid[i]);
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser3d_
 * c program to link with fortran 90
 * This program writes the xsize, ysize, zsize and z values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   nx    - x dimension 
 *   ny    - y dimension 
 *   nz    - z dimension 
 *   threshold - initial threshold
 *   z   - z array
 ************************************************************/

int java_mser3d_(name, nt, time, nx, ny, nz, threshold, grid) 
  char *name;
  int *nt;
  double time[];
  int nx[];
  int ny[];
  int nz[]; 
  double *threshold;
  double grid[];
{
   int sockfd;
   int ntsize = *nt;
   int name_length, index, i, j, k, size, tot;
   double thre = *threshold;
   float thresh = (float)thre;
   int command = 1;

   sockfd = initialize2();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, ntsize);     /* send no of time */
   write_float4(sockfd, thresh);

   tot = 0;
   for (i=0; i<ntsize; i++) {
       tot += (nx[i] * ny[i] * nz[i]);
   }

   write_int4(sockfd, tot); 

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write_float4(sockfd, (float)time[i]);
      write_int4(sockfd, nx[i]);
      write_int4(sockfd, ny[i]);
      write_int4(sockfd, nz[i]);

      size = nx[i] * ny[i] * nz[i];

      /* grid value */
      for (j=0; j<size; j++) {
         write_float4(sockfd, (float)grid[index]);
         index++;
      }   
   }
   
   close(sockfd);
   return 0;
}

int java_ser3d(char *name, double time, int nx, int ny, int nz, double threshold, double grid[]) {
   return java_ser3d_(name, &time, &nx, &ny, &nz, &threshold, grid);
}


int java_mser3d(char *name, int nt, double time[], int nx[], int ny[], int nz[], double threshold, double grid[]) {
   return java_mser3d_(name, &nt, time, nx, ny, nz, &threshold, grid);
}

/* sockio.c */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h> 

#include "jser.h"

/******************************************************************
 * initialize the socket
 * return socket descriptor. If failed return -1
 ******************************************************************/
int initialize()
{
   int sockfd;

   char *hostname;
   char *rawport;
   int port;

   struct  hostent *h;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return -1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   if ((h=gethostbyname(hostname)) == NULL) {  /* get the host info */
      herror("gethostbyname");
      return -1;
   }

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = 
                    inet_addr(inet_ntoa(*((struct in_addr *)h->h_addr)));
   serv_addr.sin_port = htons(port);

   /*
   printf("Host name  : %s\n", h->h_name);
   printf("IP Address : %s\n",inet_ntoa(*((struct in_addr *)h->h_addr)));
   */

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return -1;
   }

   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return -1;
   }

   return sockfd;

}



/******************************************************************
 * initialize the socket to send the data to the isosurface server
 * return socket descriptor. If failed return -1
 ******************************************************************/
int initialize2()
{
   int sockfd;

   char *hostname;
   char *rawport;
   int port;

   struct  hostent *h;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("ISOHOST");
   rawport = getenv("ISOPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return -1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   if ((h=gethostbyname(hostname)) == NULL) {  /* get the host info */
      herror("gethostbyname");
      return -1;
   }

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = 
                    inet_addr(inet_ntoa(*((struct in_addr *)h->h_addr)));
   serv_addr.sin_port = htons(port);

   /*
   printf("Host name  : %s\n", h->h_name);
   printf("IP Address : %s\n",inet_ntoa(*((struct in_addr *)h->h_addr)));
   */

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return -1;
   }

   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return -1;
   }

   return sockfd;

}


#include <stdio.h>
#include "jser.h"

test_java_ser() {
   char *name = "test";
   double time = 10.0;
   static double x[] = { 0.0, 1.0, 2.0, 3.0, 4.0 };
   static double fx[] = { 1.0, 2.0, 3.0, 4.0, 5.0 };
   java_ser(name, time, x, fx, 5);
}

test_java_mser() {      
   char *name = "test";
   int nt = 2;
   static double time[] = { 1.0, 2.0 };
   static double x[] = { 0.0, 1.0, 2.0, 3.0, 4.0,
                         0.0, 1.0, 2.0, 3.0, 4.00009 };
   static double fx[] = { 1.0, 2.0, 3.0, 4.0, 5.0,
                          5.0, 4.0, 3.0, 2.0, 2.0, };
   static int nx[] = { 5, 5 };
   java_mser(name, nt, time, x, fx, nx);
}

test_java_cmser() {      
   char *name = "test2";
   int nt = 2;
   static double time[] = { 1.0, 2.0 };
   static double x[] = { 0.0, 1.0, 2.0, 3.0, 4.0,
                         0.0, 1.0, 2.0, 3.0, 4.00009 };
   static double fx[] = { 1.0, 2.0, 3.0, 4.0, 5.0,
                          5.0, 4.0, 3.0, 2.0, 2.0, };
   static double cx[] = { 1.0, 2.0, 3.0, 4.0, 5.0,
                          5.0, 4.0, 3.0, 2.0, 2.0, };
   static int nx[] = { 5, 5 };
   java_cmser(name, nt, time, x, fx, cx, nx);
}


test_java_ser2d() {
   char *name = "test";
   double time = 10.0;
   static double x[] = { 0.0, 0.0, 0.0, 1.0, 1.0, 1.0 };
   static double y[] = { 0.0, 1.0, 2.0, 0.0, 1.0, 2.0 };
   static double fx[] = { 1.0, 2.0, 2.0, 4.0, 5.0, 1.0 };
   java_ser2d(name, time, x, y, fx, 6);
}

test_java_mser2d() {      
   char *name = "test";
   int nt = 2;
   static double time[] = { 1.0, 2.0 };
   static double x[] = { 0.0, 0.0, 0.0, 1.0, 1.0, 1.0,
                         0.0, 0.0, 0.0, 1.0, 1.0, 1.0 };
   static double y[] = { 0.0, 1.0, 2.0, 0.0, 1.0, 2.0,
                         0.0, 1.0, 2.0, 0.0, 1.0, 2.0 };
   static double fx[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 
                          6.0, 5.0, 4.0, 3.0, 2.0, 2.0, };
   static int nx[] = { 6, 6 };
   java_mser2d(name, nt, time, x, y, fx, nx);
}

test_java_bser2d() {
   char *name = "test";
   double time = 10.0;
   static double x[] = { 0.0, 1.0 };
   static double y[] = { 0.0, 1.0, 2.0 };
   static double fx[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 1.0 };
   java_bser2d(name, time, x, 2, y, 3, fx);
   
}

test_java_bmser2d() {
   char *name = "test";
   static double time[] = { 1.0,  2.0 };
   static double x[] = { 0.0, 1.0, 0.0, 1.0, 2.0 };
   static int nx[] = { 2, 3 };
   static double y[] = { 0.0, 1.0, 2.0, 0.0, 1.0, 2.0 };
   static int ny[] = { 3, 3 };
   static double fxy[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 1.0,
        1.0, 2.0, 3.0, 4.0, 5.0, 1.0, 1.0, 2.0, 3.0  };
   java_bmser2d(name, 2, time, x, nx, y, ny, fxy);
}

test_java_bbser2d() {
   char *name = "test";
   double time = 10.0;
   static double bb[] = { 0.0, 1.0, 0.0, 2.0 };
   static double fxy[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 1.0 };
   java_bbser2d(name, time, bb, 2, 3, fxy);
   
}

test_java_bbmser2d() {
   char *name = "test";
   static double time[] = { 1.0,  2.0 };
   static double bb[][4] = { 0.0, 1.0, 0.0, 2.0, 0.0, 1.0, 0.0, 2.0 };
   static int nx[] = { 2, 3 };
   static int ny[] = { 3, 3 };
   static double fxy[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 1.0,
        1.0, 2.0, 3.0, 4.0, 5.0, 1.0, 1.0, 2.0, 3.0  };
   java_bbmser2d(name, 2, time, bb, nx, ny, fxy);
}

test_java_cbbmser2d() {
   char *name = "test";
   static double time[] = { 1.0,  2.0 };
   static double bb[][4] = { 0.0, 1.0, 0.0, 2.0, 0.0, 1.0, 0.0, 2.0 };
   static int nx[] = { 2, 3 };
   static int ny[] = { 3, 3 };
   static double fxy[] = { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
        1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0  };
   static double cxy[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 1.0,
        1.0, 2.0, 3.0, 4.0, 5.0, 1.0, 1.0, 2.0, 3.0  };
   java_cbbmser2d(name, 2, time, bb, nx, ny, fxy, cxy);
}

test_java_vser() {
   char *name = "test";
   double time = 10;
   static double x[] = { 0.0, 1.0, 2.0, 3.0, 4.0 };
   static double y[] = { 0.0, 1.0, 2.0, 1.0, 2.0, 3.0 };
   static double r[] = { 1.0, 2.0, 1.0, 1.0, 2.0, 1.0 };
   static double t[] = { 2.0, 2.0, 3.0, 2.0, 2.0, 3.0 };
   java_vser(name, time, x, y, r, t, 6);
   
}

test_java_sercon() {
   char *name = "test";
   double time = 10;
   double time2 = 20;
   
   static double x[] = { 2.5, 4.0, 2.0, 0.0, 2.0, 5.0, 
                         7.0, 8.0, 6.0, 8.0, 8.0, 6.0 };
   static double y[] = { 0.0, 2.0, 4.0, 3.0, 2.0, 2.0, 
                         1.0, 2.0, 3.0, 3.0, 5.0, 5.0 };
   static double c[] = { 0.0, 2.0, 4.0, 3.0, 2.0, 5.0, 
                         6.0, 3.0, 1.0, 2.0, 3.0, 4.0 };
   static int cl[] = { 1, 2, 3, 4, 5, -1, 6, 7, 8, -1, 9, 10, 11, 12, -1 };
   java_sercon(name, time, x, y, c, 12, cl, 15);
   java_sercon(name, time2, x, y, c, 12, cl, 15);
   
}

test_java_vmser() {
   char *name = "test";
   static double time[] = { 1.0, 2.0 };
   static double x[] = { 0.0, 1.0, 2.0, 3.0, 4.0,
                         1.0, 2.0, 3.0, 4.0, 5.0 };
   static double y[] = { 0.0, 1.0, 2.0, 1.0, 2.0, 3.0,
                         1.0, 2.0, 1.0, 2.0, 3.0, 4.0 };
   static double r[] = { 1.0, 2.0, 1.0, 1.0, 2.0, 1.0,
                         1.0, 2.0, 1.0, 1.0, 2.0, 1.0 };
   static double t[] = { 2.0, 2.0, 3.0, 2.0, 2.0, 3.0,
                         1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
   static int nx[] = { 6, 6 };
   java_vmser(name, 2, time, x, y, r, t, nx);
   
}

test_java_bbser() {
   char *name = "test";
   double time = 10.0;
   static double bb[] = { 0.0, 2.0 };
   static double fx[] = { 1.0, 2.0, 3.0};
   java_bbser(name, time, bb, 3, fx);
   
}

test_java_bbmser() {
   char *name = "test";
   static double time[] = { 1.0,  2.0 };
   static double bb[][4] = { 0.0, 2.0, 0.0, 2.0 };
   static int nx[] = { 3, 3 };
   static double fx[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 1.0 };
   java_bbmser(name, 2, time, bb, nx, fx);
}

test_java_pser() {
   char *name = "test";
   double time = 10.0;

   static double x1[] = { 0.0, 1.0, 2.0, 3.0, 4.0 };
   static double fx1[] = { 1.0, 2.0, 3.0, 4.0, 5.0 };
   static double x2[] = { 4.0, 5.0, 6.0, 7.0, 8.0 };
   static double fx2[] = { 1.0, 2.0, 3.0, 4.0, 5.0 };

   java_pser(name, time, x1, fx1, 5, 2);

   /* Uncomment next line means that each processor calculates each time data.
    * Otherwise, each processor calculates some parts of the time data
    */
   /* time = 20.0; */
   java_pser(name, time, x2, fx2, 5, 2);
}

test_java_pser2d() {
   char *name = "test";
   double time = 10.0;

   /* each processor produce 3 x 5 data set */
   static double x1[] = { 0.0, 0.0, 0.0, 0.0, 0.0,
                          1.0, 1.0, 1.0, 1.0, 1.0,
                          2.0, 2.0, 2.0, 2.0, 2.0, };
   static double y1[] = { 0.0, 1.0, 2.0, 3.0, 4.0,
                          0.0, 1.0, 2.0, 3.0, 4.0,
                          0.0, 1.0, 2.0, 3.0, 4.0  };
   static double fxy1[] = { 1.0, 2.0, 3.0, 4.0, 5.0,
                            1.0, 2.0, 3.0, 3.0, 3.0,
                            1.0, 2.0, 3.0, 4.0, 5.0 };

   static double x2[] = { 2.0, 2.0, 2.0, 2.0, 2.0,
                          3.0, 3.0, 3.0, 3.0, 3.0,
                          4.0, 4.0, 4.0, 4.0, 4.0 };
   static double y2[] = { 0.0, 1.0, 2.0, 3.0, 4.0,
                          0.0, 1.0, 2.0, 3.0, 4.0,
                          0.0, 1.0, 2.0, 3.0, 4.0  };
   static double fxy2[] = { 1.0, 2.0, 3.0, 4.0, 5.0,
                            2.0, 2.0, 2.0, 1.0, 1.0,
                            1.0, 2.0, 2.0, 2.0, 3.0 };

   java_pser2d(name, time, x1, y1, fxy1, 15, 2);

   /* Uncomment next line means that each processor calculates each time data.
    * Otherwise, each processor calculates some parts of the time data
    */
   time = 20.0; 
   java_pser2d(name, time, x2, y2, fxy2, 15, 2);
}


test_java_pbbser2d() {
   char *name = "test";
   double time = 10.0;
   static double bb1[] = { 0.0, 2.0, 0.0, 4.0 };
   static double bb2[] = { 2.0, 4.0, 0.0, 4.0 };
   
   static double fxy1[] = { 1.0, 2.0, 3.0, 4.0, 5.0,
                           1.0, 2.0, 3.0, 3.0, 3.0,
                           1.0, 2.0, 3.0, 4.0, 5.0 };
   static double fxy2[] = { 1.0, 2.0, 3.0, 4.0, 5.0,
                            2.0, 2.0, 2.0, 1.0, 1.0,
                            1.0, 2.0, 2.0, 2.0, 3.0 };
   java_pbbser2d(name, time, bb1, 3, 5, fxy1, 2);
   java_pbbser2d(name, time, bb2, 3, 5, fxy2, 2);
}
